@@ -1,4 +1,5 @@
 """The main class for the unearth package."""
+
 from __future__ import annotations
 
 import atexit
@@ -6,14 +7,17 @@ import functools
 import itertools
 import os
 import pathlib
+import warnings
+from datetime import datetime
 from tempfile import TemporaryDirectory
-from typing import TYPE_CHECKING, Iterable, NamedTuple, Sequence
+from typing import TYPE_CHECKING, Any, Iterable, NamedTuple, Sequence
 from urllib.parse import urljoin
 
 import packaging.requirements
 from packaging.utils import BuildTag, canonicalize_name, parse_wheel_filename
 from packaging.version import parse as parse_version
 
+from unearth.auth import MultiDomainBasicAuth
 from unearth.collector import collect_links_from_location
 from unearth.evaluator import (
     Evaluator,
@@ -24,9 +28,10 @@ from unearth.evaluator import (
     is_equality_specifier,
     validate_hashes,
 )
+from unearth.fetchers import Fetcher
+from unearth.fetchers.sync import PyPIClient
 from unearth.link import Link
 from unearth.preparer import noop_download_reporter, noop_unpack_reporter, unpack_link
-from unearth.session import PyPISession
 from unearth.utils import LazySequence
 
 if TYPE_CHECKING:
@@ -42,6 +47,21 @@ else:
     Source = dict
 
 
+def _check_legacy_session(session: Any) -> None:
+    try:
+        from requests import Session
+    except ModuleNotFoundError:
+        return
+
+    if isinstance(session, Session):
+        warnings.warn(
+            "The legacy requests.Session is used, which is deprecated and will be removed in the next release. "
+            "Please use `httpx.Client` instead. ",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+
+
 class BestMatch(NamedTuple):
     """The best match for a package."""
 
@@ -57,26 +77,28 @@ class PackageFinder:
     """The main class for the unearth package.
 
     Args:
-        session (PyPISession|None): The session to use for the finder.
+        session: The session to use for the finder.
             If not provided, a temporary session will be created.
-        index_urls (Iterable[str]): The index URLs to search for packages.
-        find_links (Iterable[str]): The links to search for packages.
-        trusted_hosts: (Iterable[str]): The trusted hosts.
-        target_python (TargetPython): The links must match
+        index_urls: The index URLs to search for packages.
+        find_links: The links to search for packages.
+        trusted_hosts:: The trusted hosts.
+        target_python: The links must match
             the target Python
-        ignore_compatibility (bool): Whether to ignore the compatibility check
-        no_binary (Iterable[str]): The names of the packages to disallow wheels
-        only_binary (Iterable[str]): The names of the packages to disallow non-wheels
-        prefer_binary (Iterable[str]): The names of the packages to prefer binary
+        ignore_compatibility: Whether to ignore the compatibility check
+        no_binary: The names of the packages to disallow wheels
+        only_binary: The names of the packages to disallow non-wheels
+        prefer_binary: The names of the packages to prefer binary
             distributions even if newer sdist pacakges exist.
-        respect_source_order (bool): If True, packages from the source coming earlier
+        respect_source_order: If True, packages from the source coming earlier
             are more preferred, even if they have lower versions.
-        verbosity (int): The verbosity level.
+        verbosity: The verbosity level.
+        exclude_newer_than: A datetime that when provided, excludes any version
+            newer than it.
     """
 
     def __init__(
         self,
-        session: PyPISession | None = None,
+        session: Fetcher | None = None,
         *,
         index_urls: Iterable[str] = (),
         find_links: Iterable[str] = (),
@@ -88,6 +110,7 @@ class PackageFinder:
         prefer_binary: Iterable[str] = (),
         respect_source_order: bool = False,
         verbosity: int = 0,
+        exclude_newer_than: datetime | None = None,
     ) -> None:
         self.sources: list[Source] = []
         for url in index_urls:
@@ -103,23 +126,25 @@ class PackageFinder:
         self.only_binary = {canonicalize_name(name) for name in only_binary}
         self.prefer_binary = {canonicalize_name(name) for name in prefer_binary}
         self.trusted_hosts = trusted_hosts
+        _check_legacy_session(session)
         self._session = session
         self.respect_source_order = respect_source_order
         self.verbosity = verbosity
+        self.exclude_newer_than = exclude_newer_than
+        self.headers: dict[str, str] = {}
 
         self._tag_priorities = {
             tag: i for i, tag in enumerate(self.target_python.supported_tags())
         }
 
     @property
-    def session(self) -> PyPISession:
+    def session(self) -> Fetcher:
         if self._session is None:
             index_urls = [
                 source["url"] for source in self.sources if source["type"] == "index"
             ]
-            session = PyPISession(
-                index_urls=index_urls, trusted_hosts=self.trusted_hosts
-            )
+            session = PyPIClient(trusted_hosts=self.trusted_hosts)
+            session.auth = MultiDomainBasicAuth(index_urls=index_urls)
             atexit.register(session.close)
             self._session = session
         return self._session
@@ -146,8 +171,8 @@ class PackageFinder:
         """Build an evaluator for the given package name.
 
         Args:
-            package_name (str): The desired package name
-            allow_yanked (bool): Whether to allow yanked candidates.
+            package_name : The desired package name
+            allow_yanked: Whether to allow yanked candidates.
 
         Returns:
             Evaluator: The evaluator for the given package name
@@ -161,6 +186,7 @@ class PackageFinder:
             ignore_compatibility=self.ignore_compatibility,
             allow_yanked=allow_yanked,
             format_control=format_control,
+            exclude_newer_than=self.exclude_newer_than,
         )
 
     def _build_index_page_link(self, index_url: str, package_name: str) -> Link:
@@ -228,18 +254,16 @@ class PackageFinder:
         )
 
     def _find_packages(
-        self,
-        package_name: str,
-        allow_yanked: bool = False,
+        self, package_name: str, allow_yanked: bool = False
     ) -> Iterable[Package]:
         """Find all packages with the given name.
 
         Args:
-            package_name (str): The desired package name
-            allow_yanked (bool): Whether to allow yanked candidates.
+            package_name: The desired package name
+            allow_yanked: Whether to allow yanked candidates.
 
         Returns:
-            Iterable[Package]: The packages with the given name, sorted by best match.
+            The packages with the given name, sorted by best match.
         """
         evaluator = self.build_evaluator(package_name, allow_yanked)
 
@@ -247,12 +271,17 @@ class PackageFinder:
             if source["type"] == "index":
                 link = self._build_index_page_link(source["url"], package_name)
                 result = self._evaluate_links(
-                    collect_links_from_location(self.session, link), evaluator
+                    collect_links_from_location(
+                        self.session, link, headers=self.headers
+                    ),
+                    evaluator,
                 )
             else:
                 link = self._build_find_link(source["url"])
                 result = self._evaluate_links(
-                    collect_links_from_location(self.session, link, expand=True),
+                    collect_links_from_location(
+                        self.session, link, expand=True, headers=self.headers
+                    ),
                     evaluator,
                 )
             if self.respect_source_order:
